/* ARCTIC library header. To be used as a utility by interpreters and compilers.
 * Author: Louis A. Burke
 */
#ifndef ARCTIC_H
#define ARCTIC_H

#include <stdint.h>

/* encoding/decoding */
int arctic_max_expanded_size(int compressed_size);
int arctic_max_compressed_size(int expanded_size);
void arctic_expand(char *expanded, const char *compressed);
void arctic_compress(char *compressed, const char *expanded);

/* normalization */
struct ArcticIdentifier {
    const char *start; /* a pointer into the input, or null */
    int length;
};

enum ArcticImmediateKind { ARCTIC_NAME, ARCTIC_INTEGER, ARCTIC_NUMBER };

struct ArcticImmediate {
    enum ArcticImmediateKind kind;

    union {
        struct ArcticIdentifier name;
        int64_t integer;
        double number;
    };
};

struct ArcticOperation {
    struct ArcticIdentifier label;
    struct ArcticImmediate immediate;
    int opcode; /* won't be beat or nop or comm */
};

void arctic_normalize(struct ArcticOperation *ops, const char *expanded);

#endif /* ARCTIC_H */
